# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

# flake8: noqa

from stripe.api_resources.financial_connections.account import Account
from stripe.api_resources.financial_connections.account_owner import (
    AccountOwner,
)
from stripe.api_resources.financial_connections.account_ownership import (
    AccountOwnership,
)
from stripe.api_resources.financial_connections.session import Session



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

# flake8: noqa

from stripe.api_resources.error_object import ErrorObject, OAuthErrorObject
from stripe.api_resources.list_object import ListObject
from stripe.api_resources.search_result_object import SearchResultObject

from stripe.api_resources import apps
from stripe.api_resources import billing_portal
from stripe.api_resources import checkout
from stripe.api_resources import financial_connections
from stripe.api_resources import identity
from stripe.api_resources import issuing
from stripe.api_resources import radar
from stripe.api_resources import reporting
from stripe.api_resources import sigma
from stripe.api_resources import tax
from stripe.api_resources import terminal
from stripe.api_resources import test_helpers
from stripe.api_resources import treasury

from stripe.api_resources.account import Account
from stripe.api_resources.account_link import AccountLink
from stripe.api_resources.apple_pay_domain import ApplePayDomain
from stripe.api_resources.application_fee import ApplicationFee
from stripe.api_resources.application_fee_refund import ApplicationFeeRefund
from stripe.api_resources.balance import Balance
from stripe.api_resources.balance_transaction import BalanceTransaction
from stripe.api_resources.bank_account import BankAccount
from stripe.api_resources.capability import Capability
from stripe.api_resources.card import Card
from stripe.api_resources.cash_balance import CashBalance
from stripe.api_resources.charge import Charge
from stripe.api_resources.country_spec import CountrySpec
from stripe.api_resources.coupon import Coupon
from stripe.api_resources.credit_note import CreditNote
from stripe.api_resources.credit_note_line_item import CreditNoteLineItem
from stripe.api_resources.customer import Customer
from stripe.api_resources.customer_balance_transaction import (
    CustomerBalanceTransaction,
)
from stripe.api_resources.customer_cash_balance_transaction import (
    CustomerCashBalanceTransaction,
)
from stripe.api_resources.dispute import Dispute
from stripe.api_resources.ephemeral_key import EphemeralKey
from stripe.api_resources.event import Event
from stripe.api_resources.exchange_rate import ExchangeRate
from stripe.api_resources.file import File
from stripe.api_resources.file import FileUpload
from stripe.api_resources.file_link import FileLink
from stripe.api_resources.funding_instructions import FundingInstructions
from stripe.api_resources.invoice import Invoice
from stripe.api_resources.invoice_item import InvoiceItem
from stripe.api_resources.invoice_line_item import InvoiceLineItem
from stripe.api_resources.line_item import LineItem
from stripe.api_resources.login_link import LoginLink
from stripe.api_resources.mandate import Mandate
from stripe.api_resources.payment_intent import PaymentIntent
from stripe.api_resources.payment_link import PaymentLink
from stripe.api_resources.payment_method import PaymentMethod
from stripe.api_resources.payout import Payout
from stripe.api_resources.person import Person
from stripe.api_resources.plan import Plan
from stripe.api_resources.price import Price
from stripe.api_resources.product import Product
from stripe.api_resources.promotion_code import PromotionCode
from stripe.api_resources.quote import Quote
from stripe.api_resources.refund import Refund
from stripe.api_resources.reversal import Reversal
from stripe.api_resources.review import Review
from stripe.api_resources.setup_attempt import SetupAttempt
from stripe.api_resources.setup_intent import SetupIntent
from stripe.api_resources.shipping_rate import ShippingRate
from stripe.api_resources.source import Source
from stripe.api_resources.source_transaction import SourceTransaction
from stripe.api_resources.subscription import Subscription
from stripe.api_resources.subscription_item import SubscriptionItem
from stripe.api_resources.subscription_schedule import SubscriptionSchedule
from stripe.api_resources.tax_code import TaxCode
from stripe.api_resources.tax_id import TaxId
from stripe.api_resources.tax_rate import TaxRate
from stripe.api_resources.token import Token
from stripe.api_resources.topup import Topup
from stripe.api_resources.transfer import Transfer
from stripe.api_resources.usage_record import UsageRecord
from stripe.api_resources.usage_record_summary import UsageRecordSummary
from stripe.api_resources.webhook_endpoint import WebhookEndpoint



from __future__ import absolute_import, division, print_function

import pytest

import stripe


TEST_SUBSCRIPTION_ITEM_ID = "si_123"


class TestUsageRecord(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.UsageRecord.create(
            subscription_item=TEST_SUBSCRIPTION_ITEM_ID,
            quantity=5000,
            timestamp=1524182400,
            action="increment",
        )
        request_mock.assert_requested(
            "post",
            "/v1/subscription_items/%s/usage_records"
            % (TEST_SUBSCRIPTION_ITEM_ID),
        )
        assert isinstance(resource, stripe.UsageRecord)

    def test_raises_when_creating_without_subscription_item(self):
        with pytest.raises(ValueError):
            stripe.UsageRecord.create(
                quantity=5000, timestamp=1524182400, action="increment"
            )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

from stripe.api_resources.abstract import CreateableAPIResource
from stripe.api_resources.abstract import ListableAPIResource
from stripe.api_resources.abstract import UpdateableAPIResource


class PromotionCode(
    CreateableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    """
    A Promotion Code represents a customer-redeemable code for a [coupon](https://stripe.com/docs/api#coupons). It can be used to
    create multiple codes for a single coupon.
    """

    OBJECT_NAME = "promotion_code"



from __future__ import absolute_import, division, print_function

from stripe import util
from stripe.six.moves.urllib.parse import quote_plus


def custom_method(name, http_verb, http_path=None, is_streaming=False):
    if http_verb not in ["get", "post", "delete"]:
        raise ValueError(
            "Invalid http_verb: %s. Must be one of 'get', 'post' or 'delete'"
            % http_verb
        )
    if http_path is None:
        http_path = name

    def wrapper(cls):
        def custom_method_request(cls, sid, **params):
            url = "%s/%s/%s" % (
                cls.class_url(),
                quote_plus(util.utf8(sid)),
                http_path,
            )
            obj = cls._static_request(http_verb, url, params=params)

            # For list objects, we have to attach the parameters so that they
            # can be referenced in auto-pagination and ensure consistency.
            if "object" in obj and obj.object == "list":
                obj._retrieve_params = params

            return obj

        def custom_method_request_stream(cls, sid, **params):
            url = "%s/%s/%s" % (
                cls.class_url(),
                quote_plus(util.utf8(sid)),
                http_path,
            )
            return cls._static_request_stream(http_verb, url, params=params)

        if is_streaming:
            class_method_impl = classmethod(custom_method_request_stream)
        else:
            class_method_impl = classmethod(custom_method_request)

        existing_method = getattr(cls, name, None)
        if existing_method is None:
            setattr(cls, name, class_method_impl)
        else:
            # If a method with the same name we want to use already exists on
            # the class, we assume it's an instance method. In this case, the
            # new class method is prefixed with `_cls_`, and the original
            # instance method is decorated with `util.class_method_variant` so
            # that the new class method is called when the original method is
            # called as a class method.
            setattr(cls, "_cls_" + name, class_method_impl)
            instance_method = util.class_method_variant("_cls_" + name)(
                existing_method
            )
            setattr(cls, name, instance_method)

        return cls

    return wrapper



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

from stripe import util
from stripe.api_resources.abstract import APIResourceTestHelpers
from stripe.api_resources.abstract import CreateableAPIResource
from stripe.api_resources.abstract import ListableAPIResource
from stripe.api_resources.abstract import test_helpers


@test_helpers
class InboundTransfer(CreateableAPIResource, ListableAPIResource):
    """
    Use [InboundTransfers](https://stripe.com/docs/treasury/moving-money/financial-accounts/into/inbound-transfers) to add funds to your [FinancialAccount](https://stripe.com/docs/api#financial_accounts) via a PaymentMethod that is owned by you. The funds will be transferred via an ACH debit.
    """

    OBJECT_NAME = "treasury.inbound_transfer"

    @classmethod
    def _cls_cancel(
        cls,
        inbound_transfer,
        api_key=None,
        stripe_version=None,
        stripe_account=None,
        **params
    ):
        return cls._static_request(
            "post",
            "/v1/treasury/inbound_transfers/{inbound_transfer}/cancel".format(
                inbound_transfer=util.sanitize_id(inbound_transfer)
            ),
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
            params=params,
        )

    @util.class_method_variant("_cls_cancel")
    def cancel(self, idempotency_key=None, **params):
        return self._request(
            "post",
            "/v1/treasury/inbound_transfers/{inbound_transfer}/cancel".format(
                inbound_transfer=util.sanitize_id(self.get("id"))
            ),
            idempotency_key=idempotency_key,
            params=params,
        )

    class TestHelpers(APIResourceTestHelpers):
        @classmethod
        def _cls_fail(
            cls,
            id,
            api_key=None,
            stripe_version=None,
            stripe_account=None,
            **params
        ):
            return cls._static_request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/fail".format(
                    id=util.sanitize_id(id)
                ),
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            )

        @util.class_method_variant("_cls_fail")
        def fail(self, idempotency_key=None, **params):
            return self.resource._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/fail".format(
                    id=util.sanitize_id(self.resource.get("id"))
                ),
                idempotency_key=idempotency_key,
                params=params,
            )

        @classmethod
        def _cls_return_inbound_transfer(
            cls,
            id,
            api_key=None,
            stripe_version=None,
            stripe_account=None,
            **params
        ):
            return cls._static_request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/return".format(
                    id=util.sanitize_id(id)
                ),
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            )

        @util.class_method_variant("_cls_return_inbound_transfer")
        def return_inbound_transfer(self, idempotency_key=None, **params):
            return self.resource._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/return".format(
                    id=util.sanitize_id(self.resource.get("id"))
                ),
                idempotency_key=idempotency_key,
                params=params,
            )

        @classmethod
        def _cls_succeed(
            cls,
            id,
            api_key=None,
            stripe_version=None,
            stripe_account=None,
            **params
        ):
            return cls._static_request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/succeed".format(
                    id=util.sanitize_id(id)
                ),
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            )

        @util.class_method_variant("_cls_succeed")
        def succeed(self, idempotency_key=None, **params):
            return self.resource._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/succeed".format(
                    id=util.sanitize_id(self.resource.get("id"))
                ),
                idempotency_key=idempotency_key,
                params=params,
            )



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = "prv_123"


class TestReview(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Review.list()
        request_mock.assert_requested("get", "/v1/reviews")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Review)

    def test_is_retrievable(self, request_mock):
        resource = stripe.Review.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/reviews/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Review)

    def test_can_approve(self, request_mock):
        resource = stripe.Review.retrieve(TEST_RESOURCE_ID)
        resource.approve()
        request_mock.assert_requested(
            "post", "/v1/reviews/%s/approve" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Review)

    def test_can_approve_classmethod(self, request_mock):
        resource = stripe.Review.approve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v1/reviews/%s/approve" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Review)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

# flake8: noqa

from stripe.api_resources.billing_portal.configuration import Configuration
from stripe.api_resources.billing_portal.session import Session



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = "sqr_123"


class TestTransaction(object):
    def test_is_listable(self, request_mock):
        resources = stripe.sigma.ScheduledQueryRun.list()
        request_mock.assert_requested("get", "/v1/sigma/scheduled_query_runs")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.sigma.ScheduledQueryRun)

    def test_is_retrievable(self, request_mock):
        resource = stripe.sigma.ScheduledQueryRun.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/sigma/scheduled_query_runs/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.sigma.ScheduledQueryRun)



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = "ch_123"


class TestCharge(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Charge.list()
        request_mock.assert_requested("get", "/v1/charges")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Charge)

    def test_is_searchable(self, request_mock):
        resources = stripe.Charge.search(query='currency:"USD"')
        request_mock.assert_requested(
            "get", "/v1/charges/search", {"query": 'currency:"USD"'}
        )
        assert resources.total_count == 1
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Charge)

        cnt = 0
        for c in resources.auto_paging_iter():
            assert isinstance(c, stripe.Charge)
            cnt += 1

        assert cnt == 1

    def test_is_retrievable(self, request_mock):
        resource = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/charges/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Charge)

    def test_is_creatable(self, request_mock):
        resource = stripe.Charge.create(
            amount=100, currency="usd", source="tok_123"
        )
        request_mock.assert_requested("post", "/v1/charges")
        assert isinstance(resource, stripe.Charge)

    def test_is_saveable(self, request_mock):
        resource = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        resource.save()
        request_mock.assert_requested(
            "post", "/v1/charges/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Charge)

    def test_is_modifiable(self, request_mock):
        resource = stripe.Charge.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        request_mock.assert_requested(
            "post", "/v1/charges/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Charge)

    def test_can_capture(self, request_mock):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.capture()
        request_mock.assert_requested(
            "post", "/v1/charges/%s/capture" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Charge)

    def test_can_capture_classmethod(self, request_mock):
        resource = stripe.Charge.capture(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v1/charges/%s/capture" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Charge)

    def test_can_mark_as_fraudulent(self, request_mock):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.mark_as_fraudulent()
        request_mock.assert_requested(
            "post",
            "/v1/charges/%s" % charge.id,
            {"fraud_details": {"user_report": "fraudulent"}},
        )
        assert isinstance(resource, stripe.Charge)

    def test_can_mark_as_safe(self, request_mock):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.mark_as_safe()
        request_mock.assert_requested(
            "post",
            "/v1/charges/%s" % charge.id,
            {"fraud_details": {"user_report": "safe"}},
        )
        assert isinstance(resource, stripe.Charge)
